//
//  CeBitmapView.m
//  BitmapBleueMac
//
//  Created by Renaud Pradenc on 27/08/10.
//  Copyright 2010 Céroce. All rights reserved.
//

#import "CeBitmapView.h"


@implementation CeBitmapView


- (id) initWithFrame:(NSRect)frameRect
{
	if(self = [super initWithFrame:frameRect])
	{
		[self _createBitmapWithSize:frameRect.size];
		[self _paintBitmap];
	}
	
	return self;
}

- (void) dealloc
{
	[self _releaseBitmap];
	
	[super dealloc];
}


- (void) drawRect:(NSRect) dirtyRect
{
	[self _drawBitmapInCurrentContext];
}

- (void) _createBitmapWithSize:(NSSize)size
{
	width = size.width;
	height = size.height;
	
	// Réserver la mémoire pour la bitmap
	bitmap = malloc(width * height * sizeof(PixelRGBx));
	NSAssert(bitmap != NULL, @"malloc n'a pas pu réserver la mémoire pour la bitmap");
	
	// Créer le CGBitmapContext
	CGColorSpaceRef colorspace = CGColorSpaceCreateDeviceRGB();
	NSAssert(colorspace != NULL, @"Impossible de créer le colorspace");
	
	cgContext = CGBitmapContextCreate (
		bitmap,							// data
		width,							// width
		height,							// height,
		8,								// bitsPerComponent
		width*sizeof(PixelRGBx),		// bytesPerRow
		colorspace,						// colorspace
		kCGImageAlphaNoneSkipLast);		// bitmapInfo
	NSAssert(cgContext != NULL, @"Impossible de créer le CGBitmapContext");
	
	CGColorSpaceRelease(colorspace);
}

- (void) _releaseBitmap
{
	CGContextRelease(cgContext);
	free(bitmap);
}

- (void) _paintBitmap
{
	PixelRGBx *pixelPtr = bitmap;	// Pointer sur le début de la bitmap
	
	UInt32	x, y;
	for(y = 0; y < height; y++)
	{
		for(x = 0; x < width; x++)
		{
			pixelPtr->all = 0x00000000;
			pixelPtr->component.b = 255;
			
			pixelPtr++;
		}
	}
}

- (void) _drawBitmapInCurrentContext
{
	CGImageRef image = CGBitmapContextCreateImage(cgContext);
	NSAssert(image != NULL, @"Impossible de créer CGImage < CGBitmapContext");
	
	CGContextRef currentContext = [[NSGraphicsContext currentContext] graphicsPort];
	CGRect rect = CGRectMake(0, 0, width, height);
	CGContextDrawImage(currentContext, rect, image);
	
	CGImageRelease(image);
}

@end
